@@ -143,3 +143,29 @@ def _read_in_chunks(file_object, chunk_size=_CHUNK_SIZE):
         if not data:
             break
         yield data
+
+
+def bulk_cloud_download(selected_rows, sb_session_ex):
+    query = """
+          query getS3DownloadUrl($input: SaveFileInputs){
+              getS3DownloadUrl(input: $input){
+                downloadUri
+              }
+        """
+
+    variables = {"input": {"selectedRows": selected_rows}}
+
+    requests_session = requests.session()
+
+    sb_resp = requests_session.post(
+                sb_session_ex.get_graphql_url(),
+                headers=sb_session_ex.get_header(),
+                json={'query': query, 'variables': variables}
+            )
+
+    if sb_resp.status_code != 200:
+        sb_resp_json = sb_resp.json()
+        sb_session_ex.get_logger().error(sb_resp_json)
+        raise Exception("Not status 200")
+
+    return sb_resp.json()
