@@ -410,6 +410,86 @@ class SbSession:
                 raise Exception('Cloud upload failed for', filename)
         return ret
 
+    def generate_S3_download_links(self, itemid, filenames):
+        """generate list of bulk cloud file download tokens
+
+        :param itemid: ScienceBase Catalog Item ID of the Item
+        :param filenames: Filenames of the files to download
+        :return: List of tokenized S3 download links
+        """
+        download_links = []
+
+        if not self._sbSessionEx.is_logged_in():
+            print(f'{self._username} not logged into Keycloak -- cloud services not available')
+        else:
+            item = self.get_item(itemid)
+            selected_rows = []
+
+            for filename in filenames:
+                cuid = ""
+                key = ""
+                title = ""
+                useForPreview = False
+
+                if 'files' in item:
+                    for f in item['files']:
+                        if 'name' in f:
+                            if f['name'] == filename:
+                                if 'cuid' in f:
+                                    cuid = f['cuid']
+                                if 'key' in f:
+                                    key = f['key']
+                                if 'title' in f:
+                                    title = f['title']
+                                if 'useForPreview' in f:
+                                    useForPreview = f['useForPreview']
+                                break
+
+                if cuid == "":
+                    if 'facets' in item:
+                        for facet in item['facets']:
+                            if 'files' in facet:
+                                for f in facet['files']:
+                                    if f['name'] == filename:
+                                        if 'cuid' in f:
+                                            cuid = f['cuid']
+                                        if 'key' in f:
+                                            key = f['key']
+                                        if 'title' in f:
+                                            title = f['title']
+                                        if 'useForPreview' in f:
+                                            useForPreview = f['useForPreview']
+                                        break
+
+                selected_row = {'cuid': cuid, 'key': key, 'title': title, 'useForPreview': useForPreview}
+
+                if cuid is None:
+                    raise Exception('On-premise file detected: ' + filename)
+
+                selected_rows.append(selected_row)
+
+            response = self._sbSessionEx.bulk_cloud_download(selected_rows)
+            if response:
+                for uri in response['data']['getS3DownloadUrl']:
+                    download_links.append(uri['downloadUri'])
+            else:
+                raise Exception('Tokenized S3 link generation failed for ' + itemid)
+        return download_links
+
+    def download_cloud_files(self, filenames, download_links, destination='.'):
+        """download list of ScienceBase files using tokenized S3 download links
+        :param filenames: Filenames of the files to download
+        :param download_links: List of tokenized S3 download links
+        :param destination: Local destination where files are to be downloaded
+        """
+        if len(filenames) != len(download_links):
+            raise Exception('Error: number of filenames not consistent with number of download links')
+
+        for i in range(len(download_links)):
+            filename = filenames[i]
+            link = download_links[i]
+            self.download_file(link, filename, destination)
+
     def upload_file_and_create_item(self, parentid, filename, scrape_file=True):
         """Upload a file and create a new Item in ScienceBase
 
